"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vec_1 = require("./vec");
const Vec = require("./vec");
const ArrayExt = require("./arrayext");
const FMath = require("./fmath");
const invalidElementCount = "There must be (rows × cols) elements in the array";
function dimensions(m) {
    return [m[0], m[1]];
}
exports.dimensions = dimensions;
function element(m, row, column) {
    return m[column * m[0] + row + 2];
}
exports.element = element;
function create(rows, cols) {
    let res = new Array(rows * cols + 2);
    res[0] = rows;
    res[1] = cols;
    return res;
}
exports.create = create;
function fromArray(rows, cols, array) {
    if (array.length != rows * cols)
        throw RangeError(invalidElementCount);
    return [rows, cols, ...array];
}
exports.fromArray = fromArray;
function redim(m, rows, cols, pad = 0) {
    let res = new Array(rows * cols + 2);
    res[0] = rows;
    res[1] = cols;
    let r = m[0];
    for (let i = 0; i < cols; ++i)
        for (let j = 0; j < rows; ++j)
            res[i * rows + j + 2] = [i * r + j + 2] || pad;
    return res;
}
exports.redim = redim;
function zero(rows, cols) {
    let res = new Array(rows * cols + 2).fill(0);
    res[0] = rows;
    res[1] = cols;
    return res;
}
exports.zero = zero;
function identity(dim) {
    let res = zero(dim, dim);
    for (let i = 0; i < dim; ++i)
        res[i * dim + i + 2] = 1;
    return res;
}
exports.identity = identity;
function toMat3(m) {
    let res = redim(m, 3, 3);
    res[10] = 1;
    return res;
}
exports.toMat3 = toMat3;
function toMat4(m) {
    let res = redim(m, 4, 4);
    res[17] = 1;
    if (m[0] == 2)
        res[12] = 1;
    return res;
}
exports.toMat4 = toMat4;
function translation(dim, offsets) {
    let res = identity(dim);
    let lastCol = (dim - 1) * dim + 2;
    let lastRow = Math.min(offsets.length, dim - 1);
    for (let i = 0; i < lastRow; ++i)
        res[lastCol + i] = offsets[i];
    return res;
}
exports.translation = translation;
function scaling(dim, factors) {
    let res = identity(dim);
    let len = Math.min(factors.length, dim);
    for (let i = 0; i < len; ++i)
        res[i * dim + i + 2] = factors[i];
    return res;
}
exports.scaling = scaling;
function rotationX(dim, angle) {
    let res = identity(dim);
    let sina = Math.sin(angle);
    let cosa = Math.cos(angle);
    res[dim + 3] = cosa;
    res[dim + 4] = sina;
    res[2 * dim + 3] = -sina;
    res[2 * dim + 4] = cosa;
    return res;
}
exports.rotationX = rotationX;
function rotationY(dim, angle) {
    let res = identity(dim);
    let sina = Math.sin(angle);
    let cosa = Math.cos(angle);
    res[2] = cosa;
    res[4] = -sina;
    res[2 * dim + 2] = sina;
    res[2 * dim + 4] = cosa;
    return res;
}
exports.rotationY = rotationY;
function rotationZ(dim, angle) {
    let res = identity(dim);
    let sina = Math.sin(angle);
    let cosa = Math.cos(angle);
    res[2] = cosa;
    res[3] = sina;
    res[dim + 2] = -sina;
    res[dim + 3] = cosa;
    return res;
}
exports.rotationZ = rotationZ;
function perspective(left, right, bottom, top, zNear, zFar) {
    if (zNear <= 0 || zNear >= zFar)
        throw RangeError("zNear needs to be positive and smaller thatn zFar");
    let width = right - left;
    let height = top - bottom;
    let depth = zFar - zNear;
    return [4, 4,
        (2.0 * zNear) / width, 0, 0, 0,
        0, (2.0 * zNear) / height, 0, 0,
        (right + left) / width, (top + bottom) / height, -(zFar + zNear) / depth, -1,
        0, 0, -(2.0 * zFar * zNear) / depth, 0];
}
exports.perspective = perspective;
function orthographic(left, right, bottom, top, zNear, zFar) {
    let invWidth = 1.0 / (right - left);
    let invHeight = 1.0 / (top - bottom);
    let invDepth = 1.0 / (zFar - zNear);
    return [4, 4,
        2 * invWidth, 0, 0, 0,
        0, 2 * invHeight, 0, 0,
        0, 0, -2 * invDepth, 0,
        -(right + left) * invWidth, -(top + bottom) * invHeight, -(zFar + zNear) * invDepth, 1];
}
exports.orthographic = orthographic;
function lookAt(direction, up) {
    let zaxis = Vec.norm(Vec.inv(direction));
    let xaxis = Vec.norm(Vec.cross(up, zaxis));
    let yaxis = Vec.cross(zaxis, xaxis);
    return [4, 4,
        xaxis[vec_1.x], yaxis[vec_1.x], zaxis[vec_1.x], 0,
        xaxis[vec_1.y], yaxis[vec_1.y], zaxis[vec_1.y], 0,
        xaxis[vec_1.z], yaxis[vec_1.z], zaxis[vec_1.z], 0,
        0, 0, 0, 1];
}
exports.lookAt = lookAt;
function add(mat, other, out = create(mat[0], mat[1])) {
    let len = mat.length;
    if (typeof other === "number")
        for (let i = 2; i < len; ++i)
            out[i] = mat[i] + other;
    else
        for (let i = 2; i < len; ++i)
            out[i] = mat[i] + other[i];
    return out;
}
exports.add = add;
function sub(mat, other, out = create(mat[0], mat[1])) {
    let len = mat.length;
    if (typeof other === "number")
        for (let i = 2; i < len; ++i)
            out[i] = mat[i] - other;
    else
        for (let i = 2; i < len; ++i)
            out[i] = mat[i] - other[i];
    return out;
}
exports.sub = sub;
function mul(mat, other) {
    if (typeof other === "number") {
        let len = mat.length;
        let res = create(mat[0], mat[1]);
        for (let i = 2; i < len; ++i)
            res[i] = mat[i] * other;
        return res;
    }
    else {
        let n = mat[0];
        let m = mat[1];
        let q = other[0];
        let p = other[1];
        if (m !== q)
            throw RangeError(`Cannot multiply ${n}x${m} matrix with ${q}x${p} matrix.`);
        let res = create(n, p);
        // Iterate through rows and columns
        for (let i = 0; i < n; ++i)
            for (let j = 0; j < p; ++j) {
                // Sum up rows from this with columns from other matrix.
                let val = 0;
                for (let k = 0; k < m; ++k)
                    val += mat[k * n + i + 2] * other[j * q + k + 2];
                res[j * n + i + 2] = val;
            }
        return res;
    }
}
exports.mul = mul;
function transform(mat, vec) {
    let cols = mat[1];
    if (vec.length < cols)
        throw Error(`The vector should have ${cols} elements`);
    let vecm = [cols, 1, ...vec];
    return mul(mat, vecm).slice(2);
}
exports.transform = transform;
function transpose(mat) {
    let rows = mat[0], cols = mat[1];
    let res = create(cols, rows);
    let ind = 2;
    for (let i = 0; i < rows; ++i)
        for (let j = 0; j < cols; ++j)
            res[j * rows + i + 2] = mat[ind++];
    return res;
}
exports.transpose = transpose;
function determinant(mat) {
    let luMatrix = toJaggedArray(mat);
    let result = decomposeFA(mat[0], mat[1], luMatrix)[1];
    for (let i = 0; i < luMatrix.length; ++i)
        result *= luMatrix[i][i];
    return result;
}
exports.determinant = determinant;
function invert(mat) {
    let luMatrix = toJaggedArray(mat);
    let rows = luMatrix.length;
    let result = ArrayExt.clone(luMatrix);
    let perm = decomposeFA(mat[0], mat[1], luMatrix)[0];
    let b = Array(rows);
    for (let c = 0; c < rows; c++) {
        for (let r = 0; r < rows; r++)
            b[r] = c == perm[r] ? 1 : 0;
        let x = helperSolvef(luMatrix, b);
        for (let r = 0; r < rows; r++)
            result[r][c] = x[r];
    }
    return fromJaggedArray(result);
}
exports.invert = invert;
function toJaggedArray(mat) {
    let rows = mat[0], cols = mat[1];
    let res = new Array(rows);
    for (let r = 0; r < rows; ++r) {
        res[r] = new Array(cols);
        for (let c = 0; c < cols; ++c)
            res[r][c] = mat[c * rows + r + 2];
    }
    return res;
}
function fromJaggedArray(array) {
    let rows = array.length;
    let cols = array[0].length;
    let res = create(rows, cols);
    let i = 2;
    for (let c = 0; c < cols; ++c)
        for (let r = 0; r < rows; ++r)
            res[i++] = array[r][c];
    return res;
}
function decomposeFA(rows, cols, luMatrix) {
    if (rows != cols)
        throw RangeError("Cannot decompose non-square matrix");
    // set up row permutation result
    let perm = Array(rows);
    for (let i = 0; i < rows; ++i)
        perm[i] = i;
    // toggle tracks row swaps. +1 -> even, -1 -> odd. used by MatrixDeterminant
    let toggle = 1;
    for (let c = 0; c < cols - 1; c++) // each column
     {
        let colMax = Math.abs(luMatrix[c][c]); // find largest value in col j
        let pRow = c;
        for (let r = c + 1; r < rows; r++)
            if (luMatrix[r][c] > colMax) {
                colMax = luMatrix[r][c];
                pRow = r;
            }
        if (pRow != c) {
            // if largest value not on pivot, swap rows
            let rowPtr = luMatrix[pRow];
            luMatrix[pRow] = luMatrix[c];
            luMatrix[c] = rowPtr;
            // and swap perm info
            let tmp = perm[pRow];
            perm[pRow] = perm[c];
            perm[c] = tmp;
            // adjust the row-swap toggle
            toggle = -toggle;
        }
        // handle the case where the input matrix is singular
        if (luMatrix[c][c] == 0)
            luMatrix[c][c] = 0.000001;
        for (let r = c + 1; r < rows; r++) {
            luMatrix[r][c] /= luMatrix[c][c];
            for (let k = c + 1; k < cols; k++)
                luMatrix[r][k] -= luMatrix[r][c] * luMatrix[c][k];
        }
    }
    return [perm, toggle];
}
function helperSolvef(luMatrix, vector) {
    // before calling this helper, permute b using the perm array from 
    // MatrixDecompose that generated luMatrix
    let rows = luMatrix.length;
    let res = vector.slice();
    for (let r = 1; r < rows; r++) {
        let sum = res[r];
        for (let c = 0; c < r; c++)
            sum -= luMatrix[r][c] * res[c];
        res[r] = sum;
    }
    res[rows - 1] /= luMatrix[rows - 1][rows - 1];
    for (let r = rows - 2; r >= 0; r--) {
        let sum = res[r];
        for (let c = r + 1; c < rows; c++)
            sum -= luMatrix[r][c] * res[c];
        res[r] = sum / luMatrix[r][r];
    }
    return res;
}
function equals(mat, other) {
    if (mat[0] != other[0] || mat[1] != other[1])
        return false;
    let len = mat.length;
    for (let i = 0; i < len; ++i)
        if (mat[i] != other[i])
            return false;
    return true;
}
exports.equals = equals;
function approxEquals(mat, other, epsilon) {
    if (mat[0] != other[0] || mat[1] != other[1])
        return false;
    let len = mat.length;
    for (let i = 2; i < len; ++i)
        if (!FMath.approxEquals(mat[i], other[i], epsilon))
            return false;
    return true;
}
exports.approxEquals = approxEquals;
function toString(mat) {
    let res = "";
    let rows = mat[0], cols = mat[1];
    for (let r = 0; r < rows; r++) {
        res += "[ ";
        for (let c = 0; c < cols; c++)
            res += mat[c * rows + r + 2] + " ";
        res += "]\n";
    }
    return res;
}
exports.toString = toString;
function toArray(mat) {
    return mat.slice(2);
}
exports.toArray = toArray;
function toFloat32Array(mat) {
    return new Float32Array(mat.slice(2));
}
exports.toFloat32Array = toFloat32Array;
