"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * # Quaternions
 */
const FMath = require("./fmath");
const Vec = require("./vec");
const LERP_THRESHOLD = 0.95;
function ident() {
    return [1, [0, 0, 0]];
}
exports.ident = ident;
function isReal(quat) {
    return Vec.equals(quat[1], Vec.zero(3));
}
exports.isReal = isReal;
function isPure(quat) {
    return quat[0] == 0;
}
exports.isPure = isPure;
function lenSqr([s, vec]) {
    return Vec.lenSqr(vec) + (s * s);
}
exports.lenSqr = lenSqr;
function len(quat) {
    return Math.sqrt(lenSqr(quat));
}
exports.len = len;
function isNorm(quat) {
    return FMath.approxEquals(lenSqr(quat), 1);
}
exports.isNorm = isNorm;
function norm(quat) {
    let l = len(quat);
    return [quat[0] / l, Vec.div(quat[1], l)];
}
exports.norm = norm;
function fromAxisAngle(angle, axis) {
    let lensqr = Vec.lenSqr(axis);
    if (angle == 0 || lensqr == 0)
        return ident();
    let normaxis = lensqr == 1 ? axis : Vec.div(axis, Math.sqrt(lensqr));
    let halfangle = angle / 2;
    return [Math.cos(halfangle), Vec.mul(normaxis, Math.sin(halfangle))];
}
exports.fromAxisAngle = fromAxisAngle;
function toMatrix(quat) {
    let [s, [x, y, z]] = isNorm(quat) ? quat : norm(quat);
    let xx = x * x;
    let xy = x * y;
    let xz = x * z;
    let xs = x * s;
    let yy = y * y;
    let yz = y * z;
    let ys = y * s;
    let zz = z * z;
    let zs = z * s;
    return [3, 3,
        1 - 2 * (yy + zz), 2 * (xy + zs), 2 * (xz - ys),
        2 * (xy - zs), 1 - 2 * (xx + zz), 2 * (yz + xs),
        2 * (xz + ys), 2 * (yz - xs), 1 - 2 * (xx + yy)
    ];
}
exports.toMatrix = toMatrix;
function inv(quat) {
    let res = conj(quat);
    let ls = lenSqr(quat);
    if (FMath.approxEquals(ls, 1))
        return res;
    res[0] /= ls;
    Vec.div(res[1], ls, res[1]);
    return res;
}
exports.inv = inv;
function conj([s, vec]) {
    return [s, Vec.inv(vec)];
}
exports.conj = conj;
function mul([s1, [x1, y1, z1]], [s2, [x2, y2, z2]]) {
    return [
        s1 * s2 - x1 * x2 - y1 * y2 - z1 * z2,
        [
            s1 * x2 + s2 * x1 + y1 * z2 - y2 * z1,
            s1 * y2 + s2 * y1 + z1 * x2 - z2 * x1,
            s1 * z2 + s2 * z1 + x1 * y2 - x2 * y1
        ]
    ];
}
exports.mul = mul;
function rotate(quat, vec) {
    let q = isNorm(quat) ? quat : norm(quat);
    return mul(mul(q, [0, vec]), conj(q))[1];
}
exports.rotate = rotate;
function lerp([s1, v1], [s2, v2], interPos) {
    return [FMath.mix(s1, s2, interPos), Vec.mix(v1, v2, interPos)];
}
exports.lerp = lerp;
function slerp(quat1, quat2, interPos) {
    let q1 = isNorm(quat1) ? quat1 : norm(quat1);
    let q2 = isNorm(quat2) ? quat2 : norm(quat2);
    let v1 = [q1[0], ...q1[1]];
    let v2 = [q2[0], ...q2[1]];
    let dot = Vec.dot(v1, v2);
    if (dot < 0) {
        Vec.inv(v1, v1);
        dot = -dot;
    }
    if (dot > LERP_THRESHOLD)
        Vec.mix(v1, v2, interPos, v1);
    else {
        let theta0 = Math.acos(dot);
        let theta = theta0 * interPos;
        let sintheta0 = Math.sin(theta0);
        let sintheta = Math.sin(theta);
        let s1 = Math.sin(theta0 - theta) / sintheta0;
        let s2 = sintheta / sintheta0;
        Vec.mul(v1, s1, v1);
        Vec.mul(v2, s2, v2);
        Vec.add(v1, v2, v1);
    }
    let [s, x, y, z] = v1;
    return [s, [x, y, z]];
}
exports.slerp = slerp;
function equals(quat, other) {
    return quat[0] == other[0] && Vec.equals(quat[1], other[1]);
}
exports.equals = equals;
function approxEquals(quat, other, epsilon) {
    return FMath.approxEquals(quat[0], other[0], epsilon) &&
        Vec.approxEquals(quat[1], other[1], epsilon);
}
exports.approxEquals = approxEquals;
function toString(quat) {
    return `[${quat[0]} ${Vec.toString(quat[1])}]`;
}
exports.toString = toString;
