"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * # Quaternions
 */
const FMath = require("./fmath");
const Vec = require("./vec");
const LERP_THRESHOLD = 0.99;
function ident() {
    return [[0, 0, 0], 1];
}
exports.ident = ident;
function lenSqr([vec, w]) {
    return Vec.lenSqr(vec) + (w * w);
}
exports.lenSqr = lenSqr;
function len(quat) {
    return Math.sqrt(lenSqr(quat));
}
exports.len = len;
function isNorm(quat) {
    return FMath.approxEquals(lenSqr(quat), 1, 0.001);
}
exports.isNorm = isNorm;
function norm(quat) {
    let l = len(quat);
    return [Vec.div(quat[0], l), quat[1] / l];
}
exports.norm = norm;
function fromAxisAngle(axis, angle) {
    let lensqr = Vec.lenSqr(axis);
    if (angle == 0 || lensqr == 0)
        return ident();
    let normaxis = lensqr == 1 ? axis : Vec.div(axis, Math.sqrt(lensqr));
    let halfangle = angle / 2;
    return [Vec.mul(normaxis, Math.sin(halfangle)), Math.cos(halfangle)];
}
exports.fromAxisAngle = fromAxisAngle;
function toMatrix([[x, y, z], w]) {
    var xx = x * x;
    var xy = x * y;
    var xz = x * z;
    var xw = x * w;
    var yy = y * y;
    var yz = y * z;
    var yw = y * w;
    var zz = z * z;
    var zw = z * w;
    return [3, 3,
        1 - 2 * (yy + zz), 2 * (xy - zw), 2 * (xz + yw),
        2 * (xy + zw), 1 - 2 * (xx + zz), 2 * (yz - xw),
        2 * (xz - yw), 2 * (yz + xw), 1 - 2 * (xx + yy)
    ];
}
exports.toMatrix = toMatrix;
function inv([vec, w]) {
    return [vec, -w];
}
exports.inv = inv;
function conj([vec, w]) {
    return [vec, w];
}
exports.conj = conj;
function mul([uv1, w1], [uv2, w2]) {
    let vec = Vec.mul(uv1, w2);
    Vec.add(vec, Vec.mul(uv2, w1), vec);
    Vec.add(vec, Vec.cross(uv1, uv2), vec);
    let w = w1 * w2 + Vec.dot(uv1, uv2);
    return [vec, w];
}
exports.mul = mul;
function rotate(quat, vec) {
    return mul(mul(quat, [vec, 0]), conj(quat))[0];
}
exports.rotate = rotate;
function lerp([uv1, w1], [uv2, w2], interPos) {
    return norm([Vec.mix(uv1, uv2, interPos), FMath.mix(w1, w2, interPos)]);
}
exports.lerp = lerp;
function slerp([uv1, w1], [uv2, w2], interPos) {
    let v1 = [...uv1, w1];
    let v2 = [...uv2, w2];
    let dot = Vec.dot(v1, v2);
    if (dot > LERP_THRESHOLD)
        Vec.mix(v1, v2, interPos, v1);
    else {
        let theta = Math.acos(dot) * interPos;
        Vec.sub(v2, Vec.mul(v1, dot), v2);
        Vec.norm(v2, v2);
        Vec.mul(v1, Math.cos(theta), v1);
        Vec.mul(v2, Math.sin(theta), v2);
        Vec.add(v1, v2, v1);
    }
    let [x, y, z, w] = v1;
    return [[x, y, z], w];
}
exports.slerp = slerp;
function equals(quat, other) {
    return Vec.equals(quat[0], other[0]) && quat[1] == other[1];
}
exports.equals = equals;
function toString(quat) {
    return `[${Vec.toString(quat[0])} ${quat[1]}]`;
}
exports.toString = toString;
