"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FMath = require("./fmath");
/**
 * Coordinate dimensions used in the vector types.
 */
exports.x = 0;
exports.y = 1;
exports.z = 2;
exports.w = 3;
/**
 * Create a vector of the scpecified dimension with all of its components
 * initialized to zero.
 * @param dim Vector dimension.
 */
function zero(dim) {
    return unif(dim, 0);
}
exports.zero = zero;
/**
 * Initializes a vector with the same value in all components.
 * @param dim Vector dimension.
 * @param value Value for components.
 */
function unif(dim, value) {
    return new Array(dim).fill(value);
}
exports.unif = unif;
/**
 * Return a copy of the input vector in different dimension. Resizes the
 * vector and fills possible new components with the specified padding value.
 * @param vec The input vector.
 * @param dim The dimension of the new vector.
 * @param pad The value used to pad the new vector, if dimension is greater
 * than originally.
 */
function redim(vec, dim, pad = 1) {
    let res = new Array(dim);
    for (let i = 0; i < dim; ++i)
        res[i] = vec[i] || pad;
    return res;
}
exports.redim = redim;
/**
 * Returns the vector's dimension, i.e. its number of components.
 */
function dimension(vec) {
    return vec.length;
}
exports.dimension = dimension;
/**
 * Return one or more components of the vector in arbitrary order. The components
 * returned depend on the dimensions specified in the coords argument. Note that
 * the same component can occur multiple times in coords. So, it is valid to call
 * the function like this:
 *
 * swizzle ([x, x, y])
 */
function swizzle(vec, ...coords) {
    let len = coords.length;
    let res = new Array(len);
    for (let i = 0; i < len; ++i)
        res[i] = vec[coords[i]];
    return res;
}
exports.swizzle = swizzle;
/**
 * Return the lenght of a vector squared. This is faster to calculate than the
 * actual length since taking the square root is omitted. The value is useful
 * for comparing vectors' magnitudes.
 */
function lenSqr(vec) {
    let res = 0;
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        res += vec[i] * vec[i];
    return res;
}
exports.lenSqr = lenSqr;
/**
 * Return the length of a vector.
 */
function len(vec) {
    return Math.sqrt(lenSqr(vec));
}
exports.len = len;
/**
 * Invert a vector, i.e. flip the signs of all its components.
 * @param vec Vector to be inverted.
 * @param out Result vector.
 */
function inv(vec, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = -vec[i];
    return out;
}
exports.inv = inv;
/**
 * Add a vector or scalar to the input vector componentwise.
 * @param vec Input vector.
 * @param other Vector or scalar to be added.
 * @param out Result vector.
 */
function add(vec, other, out = new Array(vec.length)) {
    let len = vec.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] + other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] + other[i];
    return out;
}
exports.add = add;
/**
 * Subtract a vector or scalar from the input vector componentwise.
 * @param vec Input vector.
 * @param other Vector or scalar to be subtracted.
 * @param out Result vector.
 */
function sub(vec, other, out = new Array(vec.length)) {
    let len = vec.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] - other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] - other[i];
    return out;
}
exports.sub = sub;
/**
 * Multiply the input vector componentwise with another vector or scalar.
 * @param vec Input vector.
 * @param other Multiplier vector or scalar.
 * @param out Result vector.
 */
function mul(vec, other, out = new Array(vec.length)) {
    let len = vec.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] * other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] * other[i];
    return out;
}
exports.mul = mul;
/**
 * Divide the input vector by another vector or scalar componentwise.
 * @param vec Input vector.
 * @param other Divider vector or number.
 * @param out Result vector.
 */
function div(vec, other, out = new Array(vec.length)) {
    let len = vec.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] / other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = vec[i] / other[i];
    return out;
}
exports.div = div;
/**
 * Normalize a vector by dividing its components by its length. If the
 * vector's lenght is zero, meaning that all of its components are zero, then
 * the function throws an error.
 */
function norm(vec, out = new Array(vec.length)) {
    let l = len(vec);
    let vlen = vec.length;
    for (let i = 0; i < vlen; ++i)
        out[i] = vec[i] / l;
    return out;
}
exports.norm = norm;
/**
 * Checks if two vectors are equal, i.e. they have the same dimension and
 * exactly same component values.
 * @param vec1 First vector
 * @param vec2 Second vector
 */
function equals(vec1, vec2) {
    let len = vec1.length;
    for (let i = 0; i < len; ++i)
        if (vec1[i] != vec2[i])
            return false;
    return true;
}
exports.equals = equals;
/**
 * Checks if two vectors are approximately equal. They must have the same
 * dimension, but their components can deviate by the amount specified in the
 * epsilon parameter.
 * @param vec1 First vector
 * @param vec2 Second vector
 * @param epsilon The maximum allowed difference of any component.
 */
function approxEquals(vec1, vec2, epsilon) {
    let len = vec1.length;
    for (let i = 0; i < len; ++i)
        if (!FMath.approxEquals(vec1[i], vec2[i], epsilon))
            return false;
    return true;
}
exports.approxEquals = approxEquals;
/**
 * Return the dot product of two vectors. The vectors must have the same dimension,
 * otherwise an error is thrown. The result of the dot product is a scalar.
 * @param vec Input vector.
 * @param other Other vector of the dot product.
 */
function dot(vec, other) {
    let res = 0;
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        res += vec[i] * other[i];
    return res;
}
exports.dot = dot;
/**
 * Return the cross product of two vectors. The vectors must be 3-dimensional.
 * @param vec Input vector.
 * @param other The other vector of the cross product.
 * @param out Result vector.
 */
function cross(vec, other, out = new Array(3)) {
    out[exports.x] = vec[exports.y] * other[exports.z] - vec[exports.z] * other[exports.y];
    out[exports.y] = vec[exports.z] * other[exports.x] - vec[exports.x] * other[exports.z];
    out[exports.z] = vec[exports.x] * other[exports.y] - vec[exports.y] * other[exports.x];
    return out;
}
exports.cross = cross;
/**
 * Run the components of the vector through the {@link Math.abs} function.
 * @param vec Input vector.
 */
function abs(vec) {
    return vec.map(Math.abs);
}
exports.abs = abs;
/**
 * Run the components of the vector through the {@link Math.floor} function.
 * @param vec Input vector.
 */
function floor(vec) {
    return vec.map(Math.floor);
}
exports.floor = floor;
/**
 * Run the components of the vector through the {@link Math.ceil} function.
 * @param vec Input vector.
 */
function ceil(vec) {
    return vec.map(Math.ceil);
}
exports.ceil = ceil;
/**
 * Run the components of the vector through the {@link Math.round} function.
 * @param vec Input vector.
 */
function round(vec) {
    return vec.map(Math.round);
}
exports.round = round;
/**
 * Run the components of the vector through the {@link FMath.fract} function.
 * @param vec Input vector.
 */
function fract(vec) {
    return vec.map(FMath.fract);
}
exports.fract = fract;
/**
 * Calculate the minimum of two vectors componentwise.
 * @param vec The input vector.
 * @param other The vector to compare with.
 * @param out Result vector.
 */
function min(vec, other, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.min(vec[i], other[i]);
    return out;
}
exports.min = min;
/**
 * Calculate the maximum of two vectors componentwise.
 * @param vec The input vector.
 * @param other The vector to compare with.
 * @param out Result vector.
 */
function max(vec, other, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.max(vec[i], other[i]);
    return out;
}
exports.max = max;
/**
 * Clamp the components of a vector to a given range.
 * @param vec The input vector.
 * @param min Minimum component value.
 * @param max Maximum component value.
 * @param out Result vector.
 */
function clamp(vec, min, max, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.clamp(vec[i], min, max);
    return out;
}
exports.clamp = clamp;
/**
 * Calculate the interpolated vector in a given position [0, 1].
 * @param vec The input vector.
 * @param other The vector to be interpolated to.
 * @param interPos The position between 0 and 1, zero representing v and one other.
 * @param out Result vector.
 */
function mix(vec, other, interPos, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.mix(vec[i], other[i], interPos);
    return out;
}
exports.mix = mix;
/**
 * Return a vector of zeros and ones depending if the input vector components are
 * greater or less than the edge value.
 * @param vec The input vector.
 * @param edge The edge to which the components are compared.
 * @param out Result vector.
 */
function step(vec, edge, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.step(vec[i], edge);
    return out;
}
exports.step = step;
/**
 * Return a vector with components in range [0, 1] depending on how close the
 * input vector values are to the lower and upper edge value. If a component is
 * less that lower edge it gets value 0. Conversely, if it is greater than the
 * upper edge, it gets value 1. If a component is between lower and upper edge,
 * its value is smoothly interpolated between zero and one.
 * @param vec The input vector.
 * @param edgeLower The lower edge to which the components are compared.
 * @param edgeUpper The upper edge to which the components are compared.
 * @param out Result vector.
 */
function smoothStep(vec, edgeLower, edgeUpper, out = new Array(vec.length)) {
    let len = vec.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.smoothStep(vec[i], edgeLower, edgeUpper);
    return out;
}
exports.smoothStep = smoothStep;
/**
 * Returns the string representation of a vector formatted like this: [x y z]
 */
function toString(vec) {
    return "[" + vec.join(" ") + "]";
}
exports.toString = toString;
